/*
 * Copyright (c) 2024 VMware Carbon Black, all rights reserved
 */

package consumer

import (
	"fmt"
	"testing"

	"carbonblack.com/protocols/psc/devicelastcontacttime"
	ks "github.com/aws/aws-sdk-go/service/kinesis"
	message "github.com/pact-foundation/pact-go/v2/message/v4"
	"github.com/rs/zerolog"
	"github.com/rs/zerolog/log"
	kc "github.com/vmware/vmware-go-kcl/clientlibrary/interfaces"
	"gitlab.bit9.local/psc/platform/ems/hello/last-contact-updater/internal/publisher"
	"google.golang.org/protobuf/proto"
)

type mockCheckPointer struct {
	kc.IRecordProcessorCheckpointer
}

func (mock mockCheckPointer) Checkpoint(_ *string) error {
	return nil
}

type ErrorHook struct {
	hasError bool
	msg      string
}

func (h *ErrorHook) Run(_ *zerolog.Event, level zerolog.Level, msg string) {
	if level == zerolog.ErrorLevel {
		h.hasError = true
		h.msg = msg
	}
}

func TestLCTConsumer(test *testing.T) {
	provider := createProvider(test)
	lctConsumer, errorHook := createConsumer()

	lct := &devicelastcontacttime.DeviceLastContactTime{
		OrgKey:               "7ZP4H5XR",
		DeviceId:             121,
		LastContactTimestamp: 121,
		DeregisterAfterTime:  121,
	}
	protobufBytes, err := proto.Marshal(lct)
	if err != nil {
		test.Fatalf("Error creating lct message input %v", err)
	}

	handler := message.AsynchronousConsumer(func(msg message.AsynchronousMessage) error {
		lctConsumer.ProcessRecords(&kc.ProcessRecordsInput{
			Records: []*ks.Record{{Data: msg.Contents}}, Checkpointer: mockCheckPointer{},
		})
		if errorHook.hasError {
			return fmt.Errorf("error msg = %s", errorHook.msg)
		}
		return nil
	})

	_ = provider.AddAsynchronousMessage().Given("a healthy service").ExpectsToReceive("lct message").
		WithContent("*/*", protobufBytes).ConsumedBy(handler).Verify(test)
}

func TestLCTConsumerNegative(test *testing.T) {
	provider := createProvider(test)
	lctConsumer, errorHook := createConsumer()

	expectsErrorhandler := message.AsynchronousConsumer(func(msg message.AsynchronousMessage) error {
		lctConsumer.ProcessRecords(&kc.ProcessRecordsInput{
			Records: []*ks.Record{{Data: msg.Contents}}, Checkpointer: mockCheckPointer{},
		})
		if !errorHook.hasError {
			return fmt.Errorf("didn't get expected parsing error")
		}
		return nil
	})

	_ = provider.AddAsynchronousMessage().Given("a healthy service").ExpectsToReceive("malformed lct message").
		WithContent("*/*", []byte("DEFINETLYNOTGOODLOL")).ConsumedBy(expectsErrorhandler).Verify(test)
}

func createProvider(test *testing.T) *message.AsynchronousPact {
	provider, err := message.NewAsynchronousPact(message.Config{
		Consumer: "ems-last-contact-updater",
		Provider: "ems-device-hello",
	})
	if err != nil {
		test.Fatalf("Error creating mock provider %v", err)
	}

	return provider
}

func createConsumer() (lctRecordProcessor, *ErrorHook) {
	errorHook := ErrorHook{}
	hookedLogger := log.Hook(&errorHook)

	lctConsumer := lctRecordProcessor{publishers: []*publisher.Publisher{}, log: &hookedLogger}
	return lctConsumer, &errorHook
}
